﻿using System;
using System.IO;
using System.Linq;
using LibreriaIndirizziIP;

/*
 *    Studentessa: Maria Rosaria VINCIGUERRA
 *      Matricola: 12267
 *      
 *         Classe: 4AIN
 *           A.S.: 2022/2023
 *        
 *           Data: 11/08/2023
 *        
 *      Obiettivo: Creare un file CSV importabile su draw.io per la generazione
 *                 automatica di un diagramma di reti. 
 *      
 *    Descrizione: Dati in input N reti e un numero di host, per ciascuna di esse, scrivere
 *                 con l'ausilio della classe StreamWriter in un file txt i dati
 *                 relativi ai seguenti campi:
 *                 
 *                      - entry         -> un numero che identifica l'elemento grafico
 *                      - type          -> una stringa che determina lo stile dell'elemento
 *                      - host          -> una stringa che verrà utilizzata come label dell'elemento
 *                      - labeltype     -> una stringa che identifica il tipo di label, diverso in base allo stile
 *                      - width         -> un numero double che indica la larghezza dell'elemento
 *                      - height        -> un numero double che indica l'altezza dell'elemento
 *                      - connected     -> una stringa che identifica l'elemento/gli elementi connesso/i
 *                      - hidden        -> una stringa che identifica l'elemento/gli elementi connesso/i
 *                                         con collegamenti nascosti di proposito (per la label "text")
 *                      - labelRouter   -> una stringa che indica l'indirizzo ip della seconda interfaccia di un router
 *                      - network       -> una stringa contenente l'indirizzo di rete in notazione cidr
 *                      
 *                 Il diagramma di ogni singola rete sarà costituito da questi elementi grafici:
 *                 
 *                      - 1 router
 *                      - 1 switch
 *                      - 2 host (un server e un pc)     
 *                      - text (principali informazioni della rete)              
 */

namespace PianoIndirizzamento
{
    internal class PianoIndirizzamento
    {
        const string path = "..\\..\\..\\..\\";
        const string conf = "Configurazione_diagramma.txt";
        const string nome = "piano.txt";
        const int NumRetiMAX = 5;

        public static readonly double[] width = { 78, 97, 43, 78, 210 };
        public static readonly double[] height = { 53, 40, 62, 70, 117 };
        public static readonly string[] types = { "router", "switch", "server", "pc", "text" };
        public struct Rete
        {
            public int entry;
            public string type;
            public string host;
            public string labeltype;
            public double width;
            public double height;
            public string connected;
            public string hidden;
            public string labelRouter;
            public string network;
        }
        static void Main(string[] args)
        {
            StreamWriter sw = new StreamWriter(path + nome);
            StreamReader sr = new StreamReader(path + conf);
            IndirizzoIP ind = new IndirizzoIP();
            Rete[] reti = new Rete[types.Length];

            string configurazione = sr.ReadToEnd();
            sw.WriteLine(configurazione);

            int numReti;

            do
            {
                Console.Write("Inserisci il numero di reti: ");
                numReti = Convert.ToInt32(Console.ReadLine());

                if (numReti > NumRetiMAX)
                    Console.WriteLine("Si possono inserire al massimo {0} reti.\n", NumRetiMAX);

            } while (numReti <= 0 || numReti > NumRetiMAX);

            string[] connected = new string[numReti];

            sw.WriteLine("entry,type,host,labeltype,width,height,connected,hidden,labelRouter,network");
            sw.WriteLine();

            int entry = 1;
            string network = "10.0.0.0";

            for (int i = 0; i < numReti; i++)
            {
                int N;

                Console.WriteLine("\nRete n.{0}", i + 1);

                do
                {
                    Console.Write("Inserisci il numero di host: ");
                    N = Convert.ToInt32(Console.ReadLine());

                    if (N + 3 > Math.Pow(2,24))
                        Console.WriteLine("Non è possibile inserire più di 2^24 host.");

                } while (N < 0 || N + 3 > Math.Pow(2, 24));

                int h = (int)Math.Ceiling(Math.Log2(N + 3));
                int n = 32 - h;

                string networkCidr = network + "/" + n;
                string netmask = ind.ndp(ind.netmask(networkCidr));
                string broadcast = ind.ndp(ind.broadcast(ind.binario(network), ind.binario(netmask)));
                string gateway = Gateway(broadcast);
                string primoInd = Primo(network);
                string ultimoInd = Ultimo(N, network, broadcast, gateway);
                string descrizione = $"Rete n.{i + 1}<br><br>Network: {network}<br>Subnet Mask: {netmask}<br>Broadcast: {broadcast}<br>Default Gateway: {gateway}<br>Numero di host: {N}";

                string[] indirizzi = { gateway, $"Switch {i + 1}", primoInd, ultimoInd, descrizione };
                
                for (int j = 0; j < reti.Length; j++)
                {
                    reti[j].entry = entry;
                    reti[j].type = types[j];
                    reti[j].host = indirizzi[j];
                    reti[j].labeltype = "label1";
                    reti[j].width = width[j];
                    reti[j].height = height[j];

                    switch (reti[j].type)
                    {
                        case "router":
                            reti[j].connected = $"{reti[j].entry + 1}";
                            reti[j].labeltype = "label2";
                            reti[j].labelRouter = $".{i + 1}";
                            connected[i] = reti[j].entry.ToString();
                            break;

                        case "switch":
                            reti[j].connected = $"\"{reti[j].entry + 1},{reti[j].entry + 2}\"";
                            reti[j].labeltype = "label3";
                            reti[j].network = $"{networkCidr}";
                            break;

                        case "server":
                            reti[j].hidden = $"{reti[j].entry + 2}";
                            break;

                        case "pc":
                            reti[j].hidden = $"{reti[j].entry + 1}";
                            break;
                    }

                    entry++;

                    sw.Write(reti[j].entry + ",");
                    sw.Write(reti[j].type + ",");
                    sw.Write(reti[j].host + ",");
                    sw.Write(reti[j].labeltype + ",");
                    sw.Write(reti[j].width + ",");
                    sw.Write(reti[j].height + ",");
                    sw.Write(reti[j].connected + ",");
                    sw.Write(reti[j].hidden + ",");
                    sw.Write(reti[j].labelRouter + ",");
                    sw.WriteLine(reti[j].network);
                }

                sw.WriteLine();
                network = Ultimo(1, broadcast, broadcast, gateway);
            }

            string c = "";

            for (int i = 0; i < numReti; i++)
                c += connected[i] + ",";

            c = c.Remove(c.Length - 1);

            int hostID = (int)Math.Ceiling(Math.Log2(3 + numReti));
            int netID = 32 - hostID;

            sw.WriteLine($"{entry},switch,Switch,label4,97,40,\"{c}\",,,192.168.1.0/{netID}");
            sw.Write($"{entry + 1},router,192.168.1.254,label2,78,53,{entry},,");

            sw.Close();
        }
        static string Gateway(string broadcast)
        {
            IndirizzoIP ind = new IndirizzoIP();

            int[] vB = ind.vettore(broadcast);
            int gw = vB[vB.Length - 1] - 1;
            vB = vB.SkipLast(1).ToArray();
            string g = "";

            for (int i = 0; i < vB.Length; i++)
               g += vB[i].ToString() + '.';

            string ris = g + gw.ToString();

            return ris;
        }
        static string Primo(string network)
        {
            IndirizzoIP ind = new IndirizzoIP();

            int[] vN = ind.vettore(network);
            int primo = vN[vN.Length - 1] + 1;
            vN = vN.SkipLast(1).ToArray();
            string p = "";

            for (int i = 0; i < vN.Length; i++)
                p += vN[i].ToString() + '.';

            string ris = p + primo.ToString();

            return ris;
        }

        static string Ultimo(int N, string network, string broadcast, string gateway)
        {
            IndirizzoIP ind = new IndirizzoIP();
            int[] vet = ind.vettore(network);
            string ip = "";

            for (int i = 0; i < N; i++)
            {
                vet[3]++;

                if (vet[3] == 256)
                {
                    vet[2]++;
                    vet[3] = 0;

                    if (vet[2] == 256)
                    {
                        vet[1]++;
                        vet[2] = 0;

                        if (vet[1] == 256)
                        {
                            vet[0]++;
                            vet[1] = 0;
                        }
                    }
                }
            }

            for (int j = 0; j < vet.Length; j++)
                ip += vet[j].ToString() + '.';

            ip = ip.Remove(ip.Length - 1);

            if (ip != broadcast && ip != gateway)
                return ip;
            else
                return Ultimo(1, ip, broadcast, gateway);
        }
    }
}
